<?php
/**
 * @category    Payment
 * @author      Global Payments Europe s.r.o. (emailgpwebpay@gpe.cz)
 */

define('TABLE_PAYMENT_GPWEBPAY_TRANSACTION', 'payment_gpwebpay_transaction');
define('GPWEBPAY_PLUGIN_VERSION', '1.0.2');
define('GPWEBPAY_PLUGIN_NAME', 'GambioGX2_GPWEBPAY');
define('COMPARE_SHOP_VERSION', 'v3.9');

defined('GM_HTTP_SERVER') or define('GM_HTTP_SERVER', HTTP_SERVER);

class gpwebpay_core{
    var $code, $title, $description, $enabled, $tmpOrders;
    var $process_cart_id;
    
    var $coo_product;
    var $coo_xtc_price;
    var $c_path = '';

    /// @brief initialize gpwebpay_checkout_page module
    function init() {
        include(DIR_FS_CATALOG.'lang/'.$_SESSION['language'].'/modules/payment/gpwebpay.php');
        include(DIR_FS_CATALOG . 'release_info.php');

        $this->code = get_class($this);
        if (version_compare($gx_version, COMPARE_SHOP_VERSION, '>=')) {
            $this->logo_url     = DIR_WS_CATALOG.'images/icons/payment/'.$this->logoFilename ;
            $this->title        = ' '.gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_TEXT_TITLE");
        } else {
            $logoTag = ($this->logoFilename) ? '<img src="'.DIR_WS_CATALOG.'images/icons/payment/'.$this->logoFilename.'" alt="GP webpay logo"/>' : '';
            $this->title        = $logoTag.' '.gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_TEXT_TITLE");
        }

        $this->description  = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_TEXT_DESCRIPTION");
        $this->info         = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_TEXT_INFO");
        $this->enabled      = ((gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_STATUS") == 'True') ? true : false);
    }

    function constant($name) {
        return (defined($name)) ? constant($name) : NULL;
    }

    function _isInstalled($c) {
        $result = xtc_db_query("SELECT count(*) FROM ".TABLE_CONFIGURATION." WHERE configuration_key = 'MODULE_PAYMENT_GPWEBPAY_PAY_STATUS'");
        $resultRow = $result->fetch_row();
        return $resultRow[0];
    }
    
    /**
     * Get url by order parameters and GP webpay configuration
     */
    function get_gpwebpay_url()
    {
        global $order, $insert_id;
        
        $response = array();
        $response['url'] = "";
        $response['error'] = false;
        $response['text'] = "";
        $rootPath = dirname(__DIR__);
        $certPath = $rootPath . "/payment/Certs/";
        include $rootPath . "/payment/Muzo/gpmuzo.php";;
        $response['url'] = "";
        $response['error'] = false;
        
        if (!empty(gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_GATEWAY_URL"))) {
            $gateway = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_GATEWAY_URL");
        } 
        else 
        {
            $response['error'] = true;
            $response['text'] = "Empty gateway URL";
        }
        
        if (!empty(gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_MERCHANT_NUMBER"))) {
            $merchantNumber = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_MERCHANT_NUMBER");
        }
        else 
        {
            $response['error'] = true;
            $response['text'] = "Empty merchant number";
        }
        
        if (!empty(gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PUBLIC_KEY_FILE"))) {
            $publicKeyFilename = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PUBLIC_KEY_FILE");
            if(!file_exists($certPath . $publicKeyFilename))
            {
                $response['error'] = true;
                $response['text'] = "Public key not found";
            }
        } 
        else 
        {
            $response['error'] = true;
            $response['text'] = "Public key is empty";
        }
        
        if (!empty(gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PRIVATE_KEY_FILE"))) {
            $privateKeyFilename = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PRIVATE_KEY_FILE");
            if(!file_exists($certPath . $privateKeyFilename))
            {
                $response['error'] = true;
                $response['text'] = "Private key not found";
            }
        } 
        else 
        {
            $response['error'] = true;
            $response['text'] = "Private key is empty";
        }
        
        if (!empty(gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PRIVATE_KEY_PASSWORD"))) {
            $privateKeyPassword = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PRIVATE_KEY_PASSWORD");
        } 
        else 
        {
            $response['error'] = true;
            $response['text'] = "Password for private key is empty";
        }
        
        if (gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_TYPE") == 1 || gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_TYPE") == 0) {
            $transferTypeInDb = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_TYPE");
            if($transferTypeInDb == 'Pay') $transferType = 1;
            else if($transferTypeInDb == 'Authorize') $transferType = 0;
        } 
        else 
        {
            $response['error'] = true;
            $response['text'] = "Transfer type not set";
        }
        
        if (!empty(gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_NUMBER_OFFSET"))) {
            $orderIdFrom = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_NUMBER_OFFSET");
        } 
        else 
        {
            $orderIdFrom = 0;
        }
        
        $amount = $order->info['total'];

        if ($_SESSION['customers_status']['customers_status_show_price_tax'] == 0 && $_SESSION['customers_status']['customers_status_add_tax_ot'] == 1)
            $amount = $amount + $order->info['tax'];
        
        $orderCurrency = $_SESSION['currency'];

        $currencyCode = array(  'CZK' => '203', //Česká koruna 
                                'EUR' => '978', //Euro
                                'GBP' => '826', //Pound sterling
                                'USD' => '840', //US dollar     
                                'HRK' => '191', //Chorvatská kuna
                                'CAD' => '124', //Canadian Dollar
                                'SEK' => '752', //Swedish Krona 
                                'RUB' => '643', //Russian ruble  
                                'PLN' => '985', //Polish zloty  
                                'RSD' => '941', //Serbian Dinar  
                                'NOK' => '578', //Norwegian Krone
                                'BGN' => '975', //Bulgarian Lev
                                'HUF' => '348', //Forint
                                'CHF' => '756', //Swiss Franc
                                'DKK' => '208', //Danish Krone
                                'RON' => '946', //Romanian new leu
                                'TRY' => '949'  //Turkish Lira
                            );
        
        $currencyCodeIso4217 = null; 
        if(strlen($currencyCode[$orderCurrency]) == 0)
        {
            $response['error'] = true;
            $response['text'] = "Currency ".$orderCurrency." is not allowed";
        }
        else
        {
            $currencyCodeIso4217 = $currencyCode[$orderCurrency];
            $amountInTransaction = $amount * 100; //always *100 (include HUF)
        }
        $gpTransactionId = $_SESSION['gpWebPayTransactionId'];
        if(empty($gpTransactionId))
        {
            $gpTransactionId = $this->get_gpwebpay_transaction_id($orderIdFrom);
        }
        $transactionDetail = $this->get_gpwebpay_transaction_by_transaction_id($gpTransactionId);
        $gpSendTransactionId = $transactionDetail['gp_send_transaction_id'];
        $_SESSION['gpWebPayTransactionId'] = $gpTransactionId;
        
        $this->update_gpwebpay_transaction($gpTransactionId, null, $insert_id);

        $returnUrl = xtc_href_link('callback/gpwebpay/checkout_page_return.php', '', 'SSL');
        $description = "Payment id ".$gpSendTransactionId;
        $userEmail = $order->customer['email_address'];
        
        $response['url'] = GpMuzo_CreateOrder(// funkce presmeruje browser s pozadavkem na server Muzo     
            $gateway, // adresa kam posilat pozadavek do Muzo
            $returnUrl, // adresa kam ma Muzo presmerovat odpoved
            $certPath . $privateKeyFilename, // soubor s privatnim klicem
            $privateKeyPassword, // heslo privatniho klice
            $merchantNumber, // cislo obchodnika
            $gpSendTransactionId, // cislo objednavky
            $amountInTransaction, // hodnota objednavky v halerich
            $currencyCodeIso4217, // kod meny, CZK..203, EUR..978, GBP..826, USD..840, povolene meny zalezi na smlouve s bankou
            $transferType, // uhrada okamzite "1", nebo uhrada az z admin rozhrani
            $insert_id, // identifikace objednavky pro obchodnika
            $description, // popis nakupu, pouze ASCII
            "X", // data obchodnika, pouze ASCII
            '', //language
            $userEmail //email
        );
        
        if(strlen($response['url']) == 0)
        {
            $response['error'] = true;
            $response['text'] = "Error in creating URL";
        }
        
        return $response;
    }
    
    function get_gpwebpay_transaction_by_transaction_id($transactionId)
    {
        $sql = 'SELECT * FROM ' . TABLE_PAYMENT_GPWEBPAY_TRANSACTION .' WHERE id="'.(int)$transactionId.'" LIMIT 1;';
        $result = xtc_db_query($sql);
        $row = xtc_db_fetch_array($result);
        
        return $row;
    }
    
    function get_gpwebpay_transaction_by_order_number($transactionId)
    {
        $sql = 'SELECT * FROM ' . TABLE_PAYMENT_GPWEBPAY_TRANSACTION .' WHERE gp_order_id="'.(int)$transactionId.'" LIMIT 1;';
        $result = xtc_db_query($sql);
        $row = xtc_db_fetch_array($result);
        
        return $row;
    }
    
    function get_gpwebpay_transaction_id($offsetPaymentNumber)
    {
        $dbArray = [];
        $dbArray['gp_state'] = "waiting for transaction id";
        $sql = 'INSERT INTO ' . TABLE_PAYMENT_GPWEBPAY_TRANSACTION .' SET gp_state = \''.$dbArray['gp_state'].'\'';
        $result = xtc_db_query($sql);
        
        $id = xtc_db_insert_id();
        $sendTransactionId = $id + $offsetPaymentNumber;
        $this->update_gpwebpay_transaction($id, "transaction id added", null, $sendTransactionId, null);
        
        return $id;
    }
    
    function update_gpwebpay_transaction($id, $state = null, $orderId = null, $sendTransactionId = null, $note = null)
    {
        $dbArray = [];
        if(!is_null($state)) $dbArray['gp_state'] = $state;
        if(!is_null($orderId)) $dbArray['gp_order_id'] = $orderId;
        if(!is_null($sendTransactionId)) $dbArray['gp_send_transaction_id'] = $sendTransactionId;
        if(!is_null($note)) $dbArray['gp_note'] = $note;
        
        if(sizeof($dbArray) > 0)
        {
            $sql = "UPDATE ".TABLE_PAYMENT_GPWEBPAY_TRANSACTION." SET updated = NOW() ";
            if(!is_null($dbArray['gp_state'])) $sql .= ", gp_state = '".$dbArray['gp_state']."' ";
            if(!is_null($dbArray['gp_order_id'])) $sql .= ", gp_order_id = '".$dbArray['gp_order_id']."' ";
            if(!is_null($dbArray['gp_send_transaction_id'])) $sql .= ", gp_send_transaction_id = '".$dbArray['gp_send_transaction_id']."' ";
            if(!is_null($dbArray['gp_note'])) $sql .= ", gp_note = '".$dbArray['gp_note']."' ";
            $sql .= "WHERE id = ".$id;
            $result = xtc_db_query($sql);
        }
    }
    
    function isResponseValid($response)
    {
        $validationResult = false;
        
        $rootPath = dirname(__DIR__);
        $certPath = $rootPath . "/payment/Certs/";
        include $rootPath . "/payment/Muzo/gpmuzo.php";
        
        $merchantOrderNumber = $response['MERORDERNUM'];
        $transactionDetail = $this->get_gpwebpay_transaction_by_order_number($merchantOrderNumber);
        $orderNumber = $response['ORDERNUMBER'];
        if($orderNumber == $transactionDetail['gp_send_transaction_id'])
        {
            $publicKeyFilename = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_PUBLIC_KEY_FILE");
            $muzoPublicKeyFile = $certPath . $publicKeyFilename;
            $merchantNumber = gpwebpay_core::constant("MODULE_PAYMENT_GPWEBPAY_PAY_MERCHANT_NUMBER");
            $md = $response['MD']; 
            $prCode = $response['PRCODE']; 
            $srCode = $response['SRCODE'];
            $resultText = $response['RESULTTEXT'];
            $validationResult = GpMuzo_ReceiveReply(// funkce zpracuje a overi zpetne presmerovani z Muzo
                $muzoPublicKeyFile, // soubor s verejnym klicem Muzo
                $orderNumber, // cislo objednavky
                $merchantOrderNumber, // identifikace objednavky pro obchodnika
                $md, // data obchodnika, pouze ASCII
                $prCode, // primarni kod
                $srCode, // sekundarni kod
                $resultText, // slovni popis chyby
                $merchantNumber);
            if($validationResult)
            {
                $this->update_gpwebpay_transaction($transactionDetail['id'], 'GP reply is valid', null, null, "Result is VALID, PR=".$prCode.", SR=".$srCode.", Result=".$resultText);
            }
            else {
                $this->update_gpwebpay_transaction($transactionDetail['id'], 'GP reply is INVALID', null, null, "Result is INVALID!, PR=".$prCode.", SR=".$srCode.", Result=".$resultText);
            }
        }
        
        return $validationResult;
    }
        
    function selection() {
        include(DIR_FS_CATALOG . 'release_info.php');
        if (!$this->_preCheck()) {
            return false;
        }
	    if (version_compare($gx_version, COMPARE_SHOP_VERSION, '>=')) {
            return array ('id' => $this->code, 'module' => $this->title, 'description' => $this->info, 'logo_url' => $this->logo_url);
        } else {
            return array ('id' => $this->code, 'module' => $this->title, 'description' => $this->info);
        }
    }

    function _preCheck()
    {
        return true;
    }
    
    function javascript_validation() {
        return false;
    }

    function pre_confirmation_check() {
        return false;
    }

    function confirmation() {
        return false;
    }
    
    function process_button() {
	    return "";
    }
    
    /// vola se pred placenim 1
    function before_process() {
        if(isset($_SESSION['tmp_oID'])) {
            $_SESSION['gpwebpay_checkout_page']['tmp_oID'] = $_SESSION['tmp_oID'];
        }
        unset($_SESSION['tmp_oID']);
        $this->tmpOrders = true;
        return true;
    }
    
    function check() {
        if (!isset ($this->_check)) {
            $check_query = xtc_db_query("SELECT configuration_value FROM ".TABLE_CONFIGURATION." WHERE configuration_key='MODULE_PAYMENT_GPWEBPAY_PAY_STATUS'");
            $this->_check = xtc_db_num_rows($check_query);
        }
        return $this->_check;
    }

    //vola se pred placenim 2
    function payment_action() {
        $urlDetail = $this->get_gpwebpay_url();
        
        if(strlen($urlDetail['url']) > 0)
        {
            $_SESSION['gpWebPayTransactionId'] = null;
            xtc_redirect($urlDetail['url']);
        }
        else {
            //redirect user and show error message
            xtc_redirect(xtc_href_link(FILENAME_CHECKOUT_PAYMENT, 'payment_error='.$urlDetail['text'].'&error=true', 'SSL'));
        }
    }
    
    function install()
    {
        $cg_id = 6; 
        $s = 1; 
        $selection = "'gm_cfg_select_option(array(\'True\', \'False\'), '";

        xtc_db_query("INSERT INTO " . TABLE_CONFIGURATION . " ( configuration_key, configuration_value, configuration_group_id, sort_order, set_function, use_function, date_added) values ('MODULE_PAYMENT_GPWEBPAY_PAY_STATUS', 'True',  '$cg_id', '".$s++."', 'gm_cfg_select_option(array(\'True\', \'False\'),', null, now())");
        xtc_db_query("INSERT INTO " . TABLE_CONFIGURATION . " ( configuration_key, configuration_value, configuration_group_id, sort_order, set_function, use_function, date_added) values ('MODULE_PAYMENT_GPWEBPAY_PAY_ORDER_STATUS_ID', '2',  '$cg_id', '".$s++."', 'xtc_cfg_pull_down_order_statuses(', 'xtc_get_order_status_name', now())");
        
        $q = "INSERT INTO ".TABLE_CONFIGURATION."
                (configuration_key, configuration_value,  configuration_group_id, sort_order, set_function, date_added)
                VALUES 
                ('MODULE_PAYMENT_GPWEBPAY_PAY_GATEWAY_URL',              '',          '$cg_id', '" . $s++ . "', '',         now()),
                ('MODULE_PAYMENT_GPWEBPAY_PAY_MERCHANT_NUMBER',                 '',           '$cg_id', '" . $s++ . "', '',         now()),
                ('MODULE_PAYMENT_GPWEBPAY_PAY_PUBLIC_KEY_FILE',                 '',           '$cg_id', '" . $s++ . "', '',         now()),
                ('MODULE_PAYMENT_GPWEBPAY_PAY_PRIVATE_KEY_FILE',                 '',           '$cg_id', '" . $s++ . "', '',         now()),
                ('MODULE_PAYMENT_GPWEBPAY_PAY_PRIVATE_KEY_PASSWORD',                 '',           '$cg_id', '" . $s++ . "', '',         now()),
                ('MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_TYPE',        'Pay',      '$cg_id', '" . $s++ . "', 'gm_cfg_select_option(array(\'Pay\', \'Authorize\'), ', now()),
                ('MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_NUMBER_OFFSET',      '0',      '$cg_id', '" . $s++ . "', '', now()),
                ('MODULE_PAYMENT_GPWEBPAY_PAY_ALLOWED',                 '',           '$cg_id', '" . $s++ . "', '',         now())";
        xtc_db_query($q);
        
        $q = "CREATE TABLE IF NOT EXISTS ".TABLE_PAYMENT_GPWEBPAY_TRANSACTION."
          (id INT(11) NOT NULL AUTO_INCREMENT,
           gp_state VARCHAR(50) NULL,
           gp_order_id VARCHAR(30) NULL,
           gp_send_transaction_id VARCHAR(30) NULL,
           gp_note VARCHAR(255) NULL,
           updated TIMESTAMP on update CURRENT_TIMESTAMP NULL,
           PRIMARY KEY (id))";
        xtc_db_query($q);
    }

    function remove() {
        xtc_db_query("DELETE FROM ".TABLE_CONFIGURATION." WHERE configuration_key IN ('".implode("', '", $this->keys())."')");
    }

    function keys() {
        $c = strtoupper($this->code);
        $keys =  array( 
            "MODULE_PAYMENT_GPWEBPAY_PAY_ORDER_STATUS_ID",
            "MODULE_PAYMENT_GPWEBPAY_PAY_STATUS",
            "MODULE_PAYMENT_GPWEBPAY_PAY_GATEWAY_URL", 
            "MODULE_PAYMENT_GPWEBPAY_PAY_MERCHANT_NUMBER",
            "MODULE_PAYMENT_GPWEBPAY_PAY_PUBLIC_KEY_FILE",
            "MODULE_PAYMENT_GPWEBPAY_PAY_PRIVATE_KEY_FILE",
            "MODULE_PAYMENT_GPWEBPAY_PAY_PRIVATE_KEY_PASSWORD",
            "MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_TYPE",
            "MODULE_PAYMENT_GPWEBPAY_PAY_PAYMENT_NUMBER_OFFSET",
            "MODULE_PAYMENT_GPWEBPAY_PAY_ALLOWED");

        return $keys;
    }
}
MainFactory::load_origin_class('gpwebpay');
